@@ -170,10 +170,10 @@ get_alias (request_t *req)
 	thread_mutex_lock (&info.alias_mutex);
 
 	while ((res = avl_traverse (info.aliases, &trav))) {
-		if (ice_strcmp (res->name->host, "whatever") == 0) {
+	  if (hostname_local (res->name->host) && hostname_local (req->host)) {
 		  if (ice_strcmp (res->name->path, req->path) == 0)
 	    		break;
-		} 
+	  } 
 	}
 
 	if (!res) {
