@@ -82,6 +82,10 @@
 #include "http.h"
 #include "pool.h"
 
+/* in microseconds */
+#define READ_RETRY_DELAY 400
+#define READ_TIMEOUT 500000
+
 extern int running;
 extern server_info_t info;
 
@@ -478,8 +482,6 @@ source_func(void *conarg)
 
 			if (mt->ping == 1)
 				mt->ping = 0;
-
-			source_sleep (source);
 		}
 
 		thread_mutex_lock (&info.double_mutex);
@@ -596,7 +598,8 @@ find_mount_with_req (request_t *req)
 	/* First look for aliases */
 	alias = get_alias (req);
 	
-	if (alias && hostname_local (req->host) && hostname_local (alias->host))
+	if (alias && hostname_local (req->host) &&
+	    hostname_local (alias->host) && (req->port == alias->port))
 	{
 		xa_debug (1, "DEBUG: Found alias [%s:%d%s] -> [%s:%d%s]", req->host, req->port, req->path,
 			   alias->host, alias->port, alias->path);
@@ -806,17 +809,17 @@ add_chunk (connection_t *con)
 			stat_add_read(&con->food.source->stats, len);
 			info.hourly_stats.read_bytes += len;
 		} else {
-			my_sleep(30000);
+			my_sleep(READ_RETRY_DELAY * 1000);
 		}
 
 		tries++;
 		
 	} while ((info.use_meta_data && (read_bytes < SOURCE_READSIZE)) 
-		 || ((!info.use_meta_data) && ((double)read_bytes < ((double)SOURCE_READSIZE*3.0)/4.0) && (tries < 10431)));
+		 || ((!info.use_meta_data) && ((double)read_bytes < ((double)SOURCE_READSIZE*3.0)/4.0) && (tries < (READ_TIMEOUT / READ_RETRY_DELAY))));
 
 	/* make sure we actually got some data, and it's not a zombie connection */
 	if (read_bytes <= 0) {
-		write_log(LOG_DEFAULT, "Didn't receive data from source after %d tries, assuming it died...", tries);
+		write_log(LOG_DEFAULT, "Didn't receive data from source after %d microseconds, assuming it died...", tries * READ_RETRY_DELAY);
 		
 		/* Set this source as pending (not connected) */
 		pending_connection (con);
@@ -975,7 +978,7 @@ write_chunk(source_t *source, connection_t *clicon)
 	}
 	
 	xa_debug (4, "DEBUG: client %d tried %d times, now has %d errors %d chunks behind source", clicon->id, i,
-		  client_errors (clicon->food.client), clicon->food.client->cid - source->cid - 1);
+		  client_errors (clicon->food.client), source->cid < clicon->food.client->cid ? source->cid+CHUNKLEN - clicon->food.client->cid : source->cid - clicon->food.client->cid);
 	
 	if (clicon->food.client->alive == CLIENT_DEAD)
 		return;
@@ -1060,7 +1063,7 @@ write_chunk_with_metadata (source_t *source, connection_t *clicon)
 	}
 	
 	xa_debug (4, "DEBUG: client %d tried %d times, now has %d errors %d chunks behind source", clicon->id, i,
-		  client_errors (clicon->food.client), clicon->food.client->cid - source->cid - 1);
+		  client_errors (clicon->food.client), source->cid < clicon->food.client->cid ? source->cid+CHUNKLEN - clicon->food.client->cid : source->cid - clicon->food.client->cid);
 	
 	if (clicon->food.client->alive == CLIENT_DEAD)
 		return;
@@ -1336,13 +1339,16 @@ get_source_with_mount (const char *mount)
 {
 	avl_traverser trav = {0};
 	connection_t *travcon;
+	char* alias;
 
 	thread_mutex_lock (&info.source_mutex);
 	
 	while ((travcon = avl_traverse (info.sources, &trav)))
 	{
-		if ((ice_strcmp (travcon->food.source->audiocast.mount, mount) == 0) ||
-		    ((mount[0] != '/') && (ice_strcmp (&mount[0], &travcon->food.source->audiocast.mount[1]) == 0)))
+		if (! (alias = strchr (travcon->food.source->audiocast.mount, '/')))
+			alias = travcon->food.source->audiocast.mount;
+		if ((ice_strcmp (alias, mount) == 0) ||
+		    ((mount[0] != '/') && (ice_strcmp (&mount[0], &alias[1]) == 0)))
 		{
 			thread_mutex_unlock (&info.source_mutex);
 			return travcon;
@@ -1540,36 +1546,6 @@ source_write_to_client (source_t *source, connection_t *clicon)
 	  write_chunk (source, clicon);
 }
 
-/* Sleep for chunksize / byterate / 10 seconds */
-void
-source_sleep (source_t *source)
-{
-	double seconds;
-	int cid;
-
-	/* Sanity checks */
-	if (source->audiocast.bitrate <= 0 || source->audiocast.bitrate > 10000) {
-		xa_debug (1, "WARNING: Source bitrate %d is all messed up", source->audiocast.bitrate);
-		return;
-	}
-
-	cid = source->cid > 0 ? source->cid - 1 : CHUNKLEN - 1;
-
-	if (source->chunk[cid].len <= 0) {
-		xa_debug (1, "WARNING: source_sleep() called with empty chunk %d", cid);
-		return;
-	}
-
-	/* seconds = (chunklen / (((bitrate * 1000) / 8) * 11)) * info.sleepratio */
-
-	seconds = (double)((double)(source->chunk[cid].len) / ((source->audiocast.bitrate * 1375.0)) * info.sleep_ratio);
-
-	xa_debug (4, "DEBUG: Source sleeping for %f seconds (optimal %f)", seconds, (double)((double)(source->chunk[cid].len) / ((source->audiocast.bitrate * 1375.0))));
-	
-	my_sleep (seconds * 1000000.0);
-	
-}
-
 void
 source_get_new_clients (source_t *source)
 {
