@@ -180,7 +180,7 @@ send_file (connection_t *con, request_t *req)
 	xa_debug (1, "DEBUG: Static filename: %s...", filename);
 	suffix = strrchr(filename, '.');
 
-	if ((ice_strlen(filename) > 3) && (suffix && (ice_strcasecmp(suffix, ".mp3") == 0)) && (stat(filename, &buf) == 0)) {
+	if ((ice_strlen(filename) > 3) && (suffix && (ice_strcasecmp(suffix, ".mp3") == 0)) && !ice_strstr(filename, "/../") && (stat(filename, &buf) == 0)) {
 		length = buf.st_size;
 
 		/* Can't use fopen anymore.. it breaks with too many open files */
