@@ -73,6 +73,7 @@
 #include "client.h"
 #include "commands.h"
 #include "relay.h"
+#include "source.h"
 
 #ifndef MSG_DONTWAIT
 #define MSG_DONTWAIT 0
@@ -177,14 +178,23 @@ timer_handle_directory_servers (time_t stime)
 {
 	avl_traverser trav = {0};
 	directory_server_t *ds;
+	connection_t *defcon = get_default_mount();
 	
 	if ((stime - info.directorylasttime) >= info.touch_freq * 60) {
-		
+		/* First xa servers, then icy servers.
+		   Simply because it's ok, and needed to double lock
+		   the xa stuff, because we're traversing multiple
+		   sources and stuff. And with icy, we can't double lock
+		   at all, because friggin yp.shoutcast.com sux ass */
 		zero_trav(&trav);
 		
 		thread_mutex_lock(&info.double_mutex);
 		thread_mutex_lock(&info.directory_mutex);
+
 		while ((ds = avl_traverse(info.d_servers, &trav))) {
+			if (ds->type == icy_e)
+				continue;
+	
 			if ((ds->id < 0) && (ds->counter > 0) 
 			    && ((stime - ds->touchtime) < (info.touch_freq * ds->counter * 60))) {
 				xa_debug(2, "DEBUG: Directory server [%s] still has %d seconds to go", ds->host, 
@@ -192,21 +202,46 @@ timer_handle_directory_servers (time_t stime)
 				continue;
 			}
 			
-			if (ds->type == icy_e) {
-				if (ds->id == -1)
-					directory_add(ds);
-				else
-					directory_touch(ds);
-			} else {
-				directory_touch_xa(ds);
-			}
-			
+			directory_touch_xa (ds);
 			ds->touchtime = stime;
 		}
 
 		thread_mutex_unlock(&info.directory_mutex);
 		thread_mutex_unlock(&info.double_mutex);
+
+		thread_mutex_lock (&info.directory_mutex);
+
+		while ((ds = avl_traverse(info.d_servers, &trav))) {
+			if (ds->type != icy_e) continue;
+	
+			if ((ds->id < 0) && (ds->counter > 0) 
+			    && ((stime - ds->touchtime) < (info.touch_freq * ds->counter * 60))) {
+				xa_debug(2, "DEBUG: Directory server [%s] still has %d seconds to go", ds->host, 
+					 (stime - ds->touchtime) - (info.touch_freq * ds->counter * 60));
+				continue;
+			}
+			
+			if (ds->id == -1) {
+				if (defcon != NULL) {
+					directory_add(ds, defcon);
+					directory_touch(ds, defcon);
+				}
+			} else {
+				directory_touch(ds, defcon);
+			}
+			ds->touchtime = stime;
+		}
+
+		thread_mutex_unlock (&info.directory_mutex);
+
 		info.directorylasttime = stime;
+	} else if (defcon == NULL) {
+		thread_mutex_lock(&info.directory_mutex);
+		while ((ds = avl_traverse(info.d_servers, &trav))) {
+			if (ds->type != icy_e) continue;
+			if (ds->id != -1) directory_remove(ds);
+		}
+		thread_mutex_unlock(&info.directory_mutex);
 	}
 }
 
@@ -701,6 +736,8 @@ udp_update_metainfo(SOCKET s, connection_t *sourcecon, connection_t *clicon)
 {
 	char buf[1024]; /* Make it fit into the mtu.. that should be safe? */
 
+	xa_debug (1, "Updating metadata for host %s", con_host (clicon));
+
 	if (!sourcecon || !clicon) {
 		xa_debug(1, "WARNING: udp_update_metainfo: called with NULL pointers");
 		return 0;
